#!/usr/bin/env python2.4
from subprocess import Popen, PIPE
from urllib import urlencode
from xml.sax.saxutils import escape
import os.path

class Node(object):
    def __init__(self, name, color=None, label=None):
        self.name = name
        self.color = color
        self.label = label
        self.committer = None

    def define(self):
        style = []
        if self.color is not None:
            style.append('fillcolor="%s"' % self.color)
            style.append('style="filled"')
        if self.committer is not None:
            style.append('label="%s\\n%s"' % (self.name, self.committer))
        if len(style) > 0:
            return '%s[%s]' % (self.name, " ".join(style))

    def __str__(self):
        return self.name

def dot_output(relations):
    defined = set()
    yield "digraph G\n"
    yield "{\n"
    for (start, end) in relations:
        if start.name not in defined:
            defined.add(start.name)
            my_def = start.define()
            if my_def is not None:
                yield "    %s;\n" % my_def
        if end.name not in defined:
            defined.add(end.name)
            my_def = end.define()
            if my_def is not None:
                yield "    %s;\n" % my_def

        yield "    %s->%s;\n" % (start.name, end.name)
    yield "}\n"

def invoke_dot(input, out_file=None):
    cmdline = ['dot', '-Tsvg']
    if out_file is not None:
        cmdline.extend(('-o', out_file))
    dot_proc = Popen(cmdline, stdin=PIPE)
    for line in input:
        dot_proc.stdin.write(line)
    dot_proc.stdin.close()
    return dot_proc.wait()
